/*
 * CachedURL.java
 *
 * Created on 22 September 2002, 21:12
 */

package wowg;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 *
 * @author  jjc
 */
public class CachedURL {
    static private Map all = new HashMap();
    static public final File cvs;
    static {
        String homeDir = null;
        try {
            homeDir = System.getProperty("user.home");
        }
        catch (Exception e) {
        }
        File home = homeDir != null ? new File(homeDir + "/WWW/"): null;
        File top = new File("c:\\WWW\\");
        cvs = (home == null || !home.exists())? top : home;
        if ( cvs == null )
            System.err.println("Failed to find CVS directory");
    }
    private URL url;
    private File file;
    
    static Writer errors(String url) {
    	try {
			File f = new File(cvs,
			url.substring("http://www.w3.org/".length())+".errors");
		//	System.err.println(f.getAbsolutePath());
			return new FileWriter(f);
    	}
    	catch (IOException ioe){
    		ioe.printStackTrace();
    		return null;
    	}
    }
    
    long lastModified = 0;
    // According to local clock.
    long lastUpdated = 0;
    // According to local clock.
    long lastChanged;
    static final private int DONTCHECKWINDOW = cvs.isDirectory()?500:3600000;
   // static final private int DONTCHECKWINDOW = 500; //3600000;
    byte data[];
    // String charset;
    IOException exception;
    private CachedURL(String url) throws MalformedURLException {
        this.url = new URL(url);
    }
    
    private CachedURL(String url, boolean inCVS ) throws IllegalArgumentException {
        if ( !url.startsWith("http://www.w3.org/") )
            throw new IllegalArgumentException();
        String part = url.substring("http://www.w3.org/".length());
        if (!(tryFile(part) || tryFile(part+".rdf") || tryFile(part+".html") || tryFile(part+".txt") ))
            throw new IllegalArgumentException();
        
    }
    private boolean tryFile(String part) {
        file = new File(cvs,part);
        return file.exists() || file.isDirectory();
    }
    
    private InputStream getInputStream() throws IOException {
        update();
        return new ByteArrayInputStream(data);
    }
    
    private boolean hasChanged(long systime)  {
        try {
            update();
        }
        catch (IOException e) {
        }
        return lastChanged > systime;
    }
    /*
    private String charset() throws IOException {
        update();
        return charset;
    }
     **/
    private void update() throws IOException {
        if (url != null )
            updateURL();
        else
            updateFile();
    }
    private void updateFile() throws IOException {
        lastUpdated = System.currentTimeMillis();
        if (
        lastModified != file.lastModified() ) {
            lastChanged = lastUpdated;
            lastModified = file.lastModified();
            if ( file.isDirectory() ) {
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                PrintWriter w = new PrintWriter(new OutputStreamWriter(bos,"ASCII"));
                w.println("<html><body><p>");
                File all[] = file.listFiles();
                for (int i = 0; i<all.length; i++) {
                    if ( all[i].getName().startsWith(".") )
                        continue;
                    w.println("<a href='"+all[i].getName()+(all[i].isDirectory()?"/":"")+"'> </a>");
                }
                w.println("</p></body></html>");
                w.close();
                data = bos.toByteArray();
            } else {
            int lg = (int)file.length();
            data = new byte[lg];
            InputStream in = new FileInputStream(file);
            in.read(data);
            in.close();
            }
        }
    }
    private void updateURL() throws IOException {
        System.err.println("Updating: " + url.toString());
        long now = System.currentTimeMillis();
        if ( now < lastUpdated + DONTCHECKWINDOW ) {
            if ( exception != null )
                throw exception;
            return;
        }
        lastUpdated = now;
        try {
            URLConnection conn = url.openConnection();
            boolean checkLastModified = conn instanceof HttpURLConnection
            && lastModified != 0;
            if ( checkLastModified ) {
                conn.setIfModifiedSince(lastModified+1000);
                conn.connect();
                if (((HttpURLConnection)conn).getResponseCode()==
                HttpURLConnection.HTTP_NOT_MODIFIED ) {
                    System.err.println("Unmodified: " + url.toString());
                    return;
                }
                lastChanged = now;
            } else {
                conn.connect();
            }
            System.err.println("Getting: " + url.toString());
            lastModified = conn.getLastModified();
            String charset = conn.getContentType();
            if ( charset.toLowerCase().indexOf("charset") != -1 ) {
                System.err.println("Ignoring charset declaration for " + url.toString());
                System.err.println("        " + charset);
            }
            
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            InputStream in = conn.getInputStream();
            byte buf[] = new byte[1024];
            while ( true )  {
                int lg = in.read(buf,0,1024);
                if ( lg == -1 )
                    break;
                bos.write(buf,0,lg);
            }
            byte data2[] = bos.toByteArray();
            if ( (!checkLastModified ) ) {
                if ( data == null || data.length != data2.length )
                    lastChanged = now;
                else {
                    for (int kk=0;kk<data.length;kk++)
                        if (data[kk]!=data2[kk]) {
                            lastChanged = now;
                            break;
                        }
                }
            }
            data = data2;
            exception = null;
        }
        catch (IOException ioe) {
            if ( exception == null ||
            ! ioe.getClass().equals(exception.getClass())) {
                lastChanged = now;
            }
            System.err.println(ioe.getMessage());
            exception = ioe;
            throw ioe;
        }
        
    }
    
    
    static InputStream open(String url) throws IOException {
        System.err.println("Opening: " + url);
        return find(url).getInputStream();
    }
    static boolean hasChanged(String url,long systime) {
        try {
            return find(url).hasChanged(systime);
        }
        catch (MalformedURLException e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
            return true;
        }
    }
    /*
    static String charset(String url) throws IOException  {
        return find(url).charset();
    }
     **/
    static private CachedURL find(String url) throws MalformedURLException {
        int hash = url.indexOf('#');
        if ( hash != -1 )
            url = url.substring(0,hash);
        CachedURL rslt = (CachedURL)all.get(url);
        if ( rslt == null ) {
            try {
                rslt = new CachedURL(url,true);
            }
            catch (IllegalArgumentException e) {
                System.err.println("Not local: " + url);
                System.err.flush();
                rslt = new CachedURL(url);
            }
            all.put(url,rslt);
        }
        return rslt;
    }
    
    static public void main(String args[]) throws Exception {
        ZipMaker.main(args);
        PrintWriter local = new PrintWriter(new FileWriter("file.list") );
        PrintWriter remote = new PrintWriter(new FileWriter("url.list") );
        Iterator i = all.values().iterator();
        while (i.hasNext()) {
            CachedURL cu = (CachedURL)i.next();
            if ( cu.file != null )
                local.println( cu.file.getPath() );
            else
                remote.println( cu.url.toString() );
        }
        local.close();
        remote.close();
    }
    
}
