/*
 * ManifestLoader.java
 *
 * Created on 16 September 2002, 21:24
 */

package wowg;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.mem.*;
import com.hp.hpl.jena.vocabulary.*;
import com.hp.hpl.jena.rdf.arp.JenaReader;
import com.hp.hpl.jena.util.iterator.*;
import java.io.*;
import java.util.*;
import org.w3c.tidy.*;
import org.w3c.dom.*;
import java.util.regex.*;

/** 
 *
 * @author  jjc
 */
public class ManifestLoader  extends WebCache {
    static final public String dir = "http://www.w3.org/2002/03owlt";
    static final public String issues = "http://www.w3.org/2001/sw/WebOnt/webont-issues";
    static final public String tontology = dir+"/testOntology";
    static final public String tschema = "http://www.w3.org/2000/10/rdf-tests/rdfcore/testSchema";
    static final public String owlns = "http://www.w3.org/2002/07/owl#";
    static final public boolean show403Errors = false;
    private Resource owl;
    void load(Model model,String uri)  {
        try {
            if ( uri.endsWith(".rdf") )
                uri =uri.substring(0,uri.length()-4 );
            System.err.println("Loading: "+uri);
            arp.read(model,open(uri),uri);
        }
        catch (RDFException e) {
            Exception ne =  e.getNestedException();
            if ( ne != null && ne instanceof SecurityException )
                throw (RuntimeException)ne;
            addError(e.getMessage());
        }
        catch (SecurityException e) {
            throw e;
        }
        catch (Exception e) {
            addError(e.getMessage());
        }
    }
    
    NodeList findAnchors(String urls) {
        try {
            Tidy tidy = new Tidy();
            tidy.setOnlyErrors(true);
            tidy.setQuiet(true);
            tidy.setShowWarnings(false);
            Document doc = tidy.parseDOM(open(urls),null);
            if (tidy.getParseErrors()!=0)
                addError("Jtidy reported " + tidy.getParseErrors() + " errors in "+ getFilename(urls) +".");
        //    if (tidy.getParseWarnings()!=0)
        //        addError("Jtidy reported " + tidy.getParseWarnings() + " warnings in "+ getFilename(urls)+".");
            return doc.getElementsByTagName("a");
        }
        /*
        catch (RDFException e) {
            Exception ne =  e.getNestedException();
            if ( ne != null && ne instanceof SecurityException )
               throw (RuntimeException)ne;
            addError(e.getMessage());
        }
         **/
        catch (SecurityException e) {
            throw e;
        }
        catch (Exception e) {
            String msg = e.getMessage();
            if ( show403Errors || msg.indexOf("response code: 403 ") == -1 )
               addError(msg);
            return null;
        }
    }
    String number(int i) {
        if (i<10)
            return "00"+i;
        if (i<100)
            return "0" + i;
        return ""+i;
    }
    boolean tryLoad(Model model,String url) {
        try {
            InputStream in = open(url);
        }
        catch (Exception e) {
            String msg = e.getMessage();
            if ( show403Errors && msg.indexOf("response code: 403 ") != -1 )
                addError(msg);
            return false;
        }
        load(model,url);
        return true;
    }
    void loadManifests(Model model) {
        NodeList top = findAnchors(dir+"/");
        if ( top != null)
            for (int i=top.getLength()-1;i>=0;i--) {
                Element a = (Element)top.item(i);
                String anch = a.getAttribute("href");
                if ( anch.length() == 0 )
                    continue;
                if ( anch.indexOf(':') != -1 )
                    continue;
                if ( anch.indexOf('/') != anch.length() -1 )
                    continue;
                if ( !pat.matcher(anch).find() )
                    continue;
                NodeList lower = findAnchors(dir+"/"+anch);
                if ( lower == null ) {
                    String url = dir+"/"+anch+"Manifest.rdf";
                        hasChanged(url,0L);
                        tryLoad(model,url);
                    for (int k=1;
                    tryLoad(model,dir+"/"+anch+"Manifest"+number(k)+".rdf");
                    k++);
                } else {
                    for (int i2=lower.getLength()-1;i2>=0;i2--) {
                        Element a2 = (Element)lower.item(i2);
                        String anch2 = a2.getAttribute("href");
                        if ( anch2.length() == 0 )
                            continue;
                        if ( anch2.indexOf(':') != -1 )
                            continue;
                        if ( !anch2.startsWith("Manifest")  )
                            continue;
                        if (!anch2.endsWith(".rdf"))
                            continue;
                        load(model,dir+"/"+anch+anch2);
                    }
                }
            }
    }
    String getText(Element a) {
        org.w3c.dom.Node n = a.getFirstChild();
        if ( n != a.getLastChild() )
            addError("Unimplemented - non trivial ManifestLoader.getText()");
        if ( n instanceof Text )
            return ((Text)n).getData();
        addError("Failed to find Text node.");
        return "Internal error in processor";
    }
    void loadIssueNames(Model model) {
        NodeList top = findAnchors(issues);
        for (int i=top.getLength()-1;i>=0;i--) {
            Element a = (Element)top.item(i);
            String anch = a.getAttribute("href");
            if ( anch.length() == 0 )
                continue;
            if ( anch.indexOf(':') != -1 )
                continue;
            if ( !anch.startsWith("#I")  )
                continue;
            String label = getText(a).trim();
            int space = label.indexOf(' ');
            int dash = label.indexOf('-');
            if ( dash != -1 && ( dash < space || space == -1 ) )
                space = dash;
            if ( space == -1 )
                continue;
            String numInLabel = label.substring(0,space);
            int hyphen = anch.indexOf('-');
            if ( hyphen == -1 ) {
                addError("Strange issue: " + anch);
                continue;
            }
            String numInAnchor = anch.substring(2,hyphen);
            if ( !numInAnchor.equals(numInLabel) ) {
                try {
                    Float.parseFloat(numInAnchor);
                    Float.parseFloat(numInLabel);
                    addError("Issue list mismatch: " + numInAnchor + " != " + numInLabel + " ( " + anch + " ).");
                }
                catch (NumberFormatException e) {
                }
                continue;
            }
            //  System.err.println("Issue: " + anch + " is +\"" + label.substring(space+1) );
            try {
                model.add(model.createResource(issues+anch),RDF.type,OTEST.Issue );
                //  model.add(model.createResource(issues+".html"+anch),RDF.type,OTEST.Issue );
                model.add(model.createResource(issues+anch),OTEST.issueNumber,label.substring(0,space) );
                //  model.add(model.createResource(issues+".html"+anch),OTEST.issueNumber,label.substring(0,space) );
                model.add(model.createResource(issues+anch),RDFS.label,label.substring(space+1) );
                //  model.add(model.createResource(issues+".html" + anch),RDFS.label,label.substring(space+1) );
            }
            catch (RDFException e) {
              //  System.err.println("Hmmm");
            }
        }
    }
    public Iterator issues() {
        Map map = new TreeMap();
        try {
            ResIterator rit = model().listSubjectsWithProperty(RDF.type,OTEST.Issue);
            while ( rit.hasNext() ) {
                Issue f = new Issue((Resource)rit.next());
                try {
                    String num = f.getNumber();
              //      System.err.println("Issue " + num);
                    int dot = num.indexOf('.');
                    int val = Integer.parseInt(num.substring(0,dot)) * 1000 
                        + Integer.parseInt(num.substring(dot+1));
                    
                    map.put(new Integer(val),f);
                }
                catch (NumberFormatException e) {
                    addError(e);
                    map.put(new Integer(0),f);
                }
            }
        }
        catch (RDFException e) {
            addError(e);
        }
        return map.values().iterator();
    }
    public Iterator features() {
        Map map = new TreeMap();
        try {
            ResIterator rit = model().listSubjectsWithProperty(RDFS.isDefinedBy,owl);
            while ( rit.hasNext() ) {
                Feature f = new Feature((Resource)rit.next());
                map.put(f.getName().toLowerCase(),f);
            }
        }
        catch (RDFException e) {
            addError(e);
        }
        return map.values().iterator();
    }
    public Model model() {
        return (Model)value();
    }
    
    JenaReader arp;
    Pattern pat;
    public ManifestLoader(String pattern) {
        pat = Pattern.compile(pattern==null?".*":pattern);
    }
    /*
    static public void main(String args[]) {
        ManifestLoader ml = new ManifestLoader();
        System.err.println(ml.getErrors());
        
    }
    */
    Object get() {
        Model model = null;
        clearErrors();
        try {
            model = new ModelMem();
            arp = new JenaReader();
            arp.setErrorHandler(new RDFErrorHandler() {
                public void warning(Exception e) {
                    addError(e.getMessage());
                }
                public void error(Exception e) {
                    addError(e.getMessage());
                }
                public void fatalError(Exception e) {
                    addError(e.getMessage());
                }
            });
            owl = model.createResource(owlns.substring(0,owlns.length()-1));
            
            load(model,owlns);
            Iterator it = new FilterIterator(
            new Filter() {
                public boolean accept(Object o) {
                    String uri = ((Resource)o).getURI();
                    return uri != null && uri.startsWith(owlns);
                }
            }, model.listSubjectsWithProperty(RDF.type)
            );
            while ( it.hasNext() ) {
                ((Resource)it.next()).addProperty(RDFS.isDefinedBy,owl);
            }
            loadManifests(model);
            load(model,tontology);
            load(model,tschema);
            loadIssueNames(model);
            Set all = new HashSet();
            final Set badIssue = new HashSet();
            StmtIterator sit = model.listStatements();
            while ( sit.hasNext() ) {
                Statement stmt = (Statement)sit.next();
                all.add(stmt.getSubject());
                all.add(stmt.getPredicate());
                if ( stmt.getObject() instanceof Resource )
                    all.add(stmt.getObject());
                
            }
            sit.close();
            Iterator jj = all.iterator();
            while ( jj.hasNext() ) {
                Resource r = (Resource)jj.next();
                if (!r.isAnon()) {
                    String uri = r.getURI();
                    int hash = uri.lastIndexOf('#');
                    if ( hash == -1 )
                        hash = uri.length();
                    int dot = uri.lastIndexOf('.',hash);
                    int slash = uri.lastIndexOf('/',hash);
                    if ( dot > slash )
                        addError("Can suffix be removed from URI: " + getFilename(uri) );
                    if ( uri.startsWith(issues) )
                        badIssue.add(uri);
                }
            }
            Model badIssueStmts = model.query(new Selector() {
                public boolean test(Statement st) {
                    return badIssue.contains(st.getObject());
                }
            });
            sit = badIssueStmts.listStatements();
            while ( sit.hasNext() ) {
                Statement stmt = (Statement)sit.next();
                System.err.println("Removing: " + stmt.toString());
                model.remove(stmt);
                String frag  = getFragment(stmt.getResource().getURI());
                Statement xstmt = model.createStatement(stmt.getSubject(), stmt.getPredicate(), model.createResource( issues + "#" + frag ) );
                
                System.err.println("Adding: " + xstmt.toString());
                model.add(xstmt);
            }
            
            
        }
        catch (Exception e) {
            addError(e);
        }
        
        return model;
    }
    
}
