/*
 * Test.java
 *
 * Created on 05 September 2002, 07:31
 */

package wowg;


import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;

/**
 *
 * @author  jjc
 */
public abstract class Testx extends Testy implements Comparable {
    
    final Resource resource;
    final Property props[];
    /** Creates a new instance of Test */
    public Testx(Resource r, Property p[]) {
        resource = r;
        props = p;
    }
    /*
    boolean escape = true;
    void setEscape(boolean x) {
        escape = x;
    }
     **/
    
    static public Testx make(Resource r) {
        try {
            Resource type = r.getProperty(RDF.type).getResource();
            String test = getFragment(type.getURI());
        //    System.err.println(type.getURI() + " ==> " + test);
            if ( test.equals("PositiveEntailmentTest") )
                return new PositiveEntailmentTest(r);
            if ( test.equals("ImportEntailmentTest") )
                return new ImportEntailmentTest(r);
            if ( test.equals("NegativeEntailmentTest") )
                return new NegativeEntailmentTest(r);
            if ( test.equals("NotOwlFeatureTest") )
                return new NotOwlFeatureTest(r);
            if ( test.equals("ConsistencyTest") )
                return new ConsistencyTest(r);
            if ( test.equals("InconsistencyTest") )
                return new InconsistencyTest(r);
            if ( test.equals("MiscellaneousTest") )
                return new MiscellaneousTest(r);
            if ( test.equals("ImportsLevelTest") )
                return new ImportsLevelTest(r);
            if ( test.equals("TrueTest") )
                return new TrueTest(r);
            if ( test.equals("OWLforOWLTest") )
                return new OWLforOWLTest(r);
        }
        catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
        }
        return null;
    }
    public String getName() {
        String uri = resource.getURI();
        String frag = getFragment(uri);
        if (frag.equals("test")) {
            int ix = uri.lastIndexOf('#');
            return uri.substring(ix-3,ix);
        } else
            return frag;
    }
    public String getTestTypeFrag() throws RDFException {
        return getFragment(uriOf(RDF.type));
    }
    
    public int compareTo(Object obj) {
        Testx other = (Testx)obj;
        //  int diff = sortOrder() - other.sortOrder();
        //  if ( diff == 0 )
        return  resource.getURI().compareTo(other.resource.getURI());
    }
    static void append(StringBuffer buf,char c) {
        buf.append(c);
    }
 
    public String getAuthor() {
        return resource.hasProperty(DC.creator)?resource.getProperty(DC.creator).getString():"anon";
    }
    public String getStatus() throws RDFException {
        return resource.hasProperty(RTEST.status)?resource.getProperty(RTEST.status).getString():null;
    }
    
    public Resource getIssuette() {
        return resource.hasProperty(OTEST.issuette)?(Resource)resource.getProperty(OTEST.issuette).getObject():null;
    }
    
    Resource getFunction() throws RDFException {
        return resource.hasProperty(OTEST.feature)?(Resource)resource.getProperty(OTEST.feature).getObject():null;
    }
    Resource getIssue() throws RDFException {
        return resource.hasProperty(RTEST.issue)?(Resource)resource.getProperty(RTEST.issue).getObject():null;
    }
    String getURIs()[] {
        String rslt[] = new String[props.length];
        for (int i=0;i<rslt.length;i++)
            rslt[i] = getURI(i);
        return rslt;
    }
    public String getURI(int i) {
        return uriOf(props[i]);
    }
    public String getTestURI() {
        return resource.getURI();
    }
    Resource getLevelR() {
        return resource.hasProperty(OTEST.level)?resource.getProperty(OTEST.level).getResource():null;
    }
    Resource getLevelR(int i) {
        if ( i >= props.length )
            return null;
        if (!resource.hasProperty(props[i]))
            return null;
        Resource r = resource.getProperty(props[i]).getResource();
        return r.hasProperty(OTEST.level)?r.getProperty(OTEST.level).getResource():null;
    }
    
    Resource[] getLevels() {
		Resource rslt[] = new Resource[props.length];
		for (int i=0;i<rslt.length;i++)
			rslt[i] = getLevelR(i);
		return rslt;
    	
    }
    
    public Resource getSize() {
        if (!resource.hasProperty(OTEST.size))
            return null;
        else
            return resource.getProperty(OTEST.size).getResource();
    }
    
    abstract public String getTestType();
    
    abstract public String getFileHeading(int i);
    
    static int xhtmlDeclLength = " xmlns=\"http://www.w3.org/1999/xhtml\"".length();
    static private String removeXhtmlDecl(String s) {
        int ix = s.indexOf(" xmlns=\"http://www.w3.org/1999/xhtml\"");
        if ( ix == -1 )
            return s;
        else
            return s.substring(0,ix) + removeXhtmlDecl(s.substring(ix+xhtmlDeclLength));
    }
    public String getDescription() {
        try {
            Statement stmt = resource.getProperty(RTEST.description);
            if (stmt==null) {
                addError("Description is missing from Manifest.");
                return "";
            }
            return removeXhtmlDecl(stmt.getString().trim());
        }
        catch (RDFException e) {
            System.err.println(resource.getURI());
            addError(e);
            return "";
        }
    }
    
    public boolean equals(Object o) {
        return o != null && o instanceof Testx && ((Testx)o).resource.equals(resource);
    }
    public int hashCode() {
        return resource.getURI().hashCode();
    }
    public Triples asTriples()  {
            return WebTriples.get(getURIs(),getDescription(),getLevels());
    }
    public int getN() {
        return props.length;
    }
     
    
    public String getFile(int i) {
        WebTriples t = (WebTriples)asTriples();
        return  t.getFile(i) ;    
    }
    String uriOf(Property p) {
        try {
            if ( p == null )
                return null;
            if (!resource.hasProperty(p)) {
                addError("Missing property in manifest: &lt;" + p.getURI() +">.");
                return null;
            }
            String uri = ((Resource)resource.getProperty(p).getObject()).getURI();
            if ( uri.lastIndexOf('.') > uri.lastIndexOf('/') )
                addError("Uri &lt;" + uri + "> has a suffix; can it be removed?");
            return uri;
        }
        catch (RDFException e) {
            addError(e);
            return "empty.rdf";
        }
    }
    String getProperty(Property p) {
        try {
            if ( !resource.hasProperty(p) )
                return "";
            return resource.getProperty(p).getString();
        }
        catch (RDFException e) {
            addError(e);
            return "";
        }
    }
}
