/*
 * Triples.java
 *
 * Created on 13 September 2002, 13:19
 */

package wowg;

import org.xml.sax.helpers.*;
import org.xml.sax.*;
import org.xml.sax.ext.*;
import java.util.*;
import com.hp.hpl.jena.rdf.arp.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.OWL;
import com.hp.hpl.jena.util.*;
import com.hp.hpl.jena.ontology.tidy.*;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.ontology.impl.*;
import com.hp.hpl.jena.rdf.model.impl.Util;

import java.io.*;

/**
 *
 * @author  jjc
 */
abstract public class Triples extends HasErrors {
	private final class N3Writer implements StatementHandler, RDFWriter {
		private StringBuffer rslt;
		final Map bnodes = new HashMap();
		final Model m;
		private N3Writer(StringBuffer rslt) {
			this.rslt = rslt;
			m = ModelFactory.createDefaultModel();
		}
		public void statement(AResource subj, AResource pred, AResource obj) {
			rslt.append(
				resource(subj, bnodes)
					+ resource(pred, bnodes)
					+ resource(obj, bnodes)
					+ ".\n");
			m.add(
				jena(subj, bnodes, m),
				(Property) jena(pred, bnodes, m).as(Property.class),
				jena(obj, bnodes, m));
		}
		public void statement(AResource subj, AResource pred, ALiteral lit) {
			rslt.append(
				resource(subj, bnodes)
					+ resource(pred, bnodes)
					+ literal(lit)
					+ ".\n");
			m.add(
				jena(subj, bnodes, m),
				(Property) jena(pred, bnodes, m).as(Property.class),
				jena(lit, m));
		}
		/* (non-Javadoc)
		 * @see com.hp.hpl.jena.rdf.model.RDFWriter#write(com.hp.hpl.jena.rdf.model.Model, java.io.Writer, java.lang.String)
		 */
		public void write(Model m, Writer w, String arg2) throws RDFException {
			Iterator it = m.listStatements();
			try {
				w.write("<pre>");
				while (it.hasNext()) {
					Statement s = (Statement) it.next();
					w.write(resource(s.getSubject(), bnodes));
					w.write(" ");
					w.write(resource(s.getPredicate(), bnodes));
					w.write(" ");
					if (s.getObject().asNode().isLiteral()) {
						w.write(literal(s.getLiteral()));
					} else {
						w.write(resource(s.getResource(), bnodes));
					}
					w.write(" .\n");
				}
				w.write("</pre>");
			} catch (IOException ioe) {
				addError(ioe);
			}

		}
		/* (non-Javadoc)
		 * @see com.hp.hpl.jena.rdf.model.RDFWriter#write(com.hp.hpl.jena.rdf.model.Model, java.io.OutputStream, java.lang.String)
		 */
		public void write(Model arg0, OutputStream arg1, String arg2)
			throws RDFException {
			write(arg0, FileUtils.asUTF8(arg1), arg2);
		}
		/* (non-Javadoc)
		 * @see com.hp.hpl.jena.rdf.model.RDFWriter#setProperty(java.lang.String, java.lang.Object)
		 */
		public Object setProperty(String arg0, Object arg1)
			throws RDFException {
			return null;
		}
		/* (non-Javadoc)
		 * @see com.hp.hpl.jena.rdf.model.RDFWriter#setNsPrefix(java.lang.String, java.lang.String)
		 */
		public void setNsPrefix(String arg0, String arg1) {

		}
		/* (non-Javadoc)
		 * @see com.hp.hpl.jena.rdf.model.RDFWriter#setErrorHandler(com.hp.hpl.jena.rdf.model.RDFErrorHandler)
		 */
		public RDFErrorHandler setErrorHandler(RDFErrorHandler arg0) {
			return null;
		}
		/* (non-Javadoc)
		 * @see com.hp.hpl.jena.rdf.model.RDFWriter#getPrefixFor(java.lang.String)
		 */
		public String getPrefixFor(String ns) {
			return null;
		}
	}
	static final String copyright =
		"Copyright World Wide Web Consortium, (Massachusetts Institute of"
			+ "Technology, European Research Consortium for Informatics and "
			+ "Mathematics, Keio University)."
			+ "All Rights Reserved."
			+ "Please see the full Copyright clause at"
			+ "<http://www.w3.org/Consortium/Legal/copyright-software.html>"
			+ "$Id: Triples.java,v 1.24 2003/04/19 05:29:59 jcarroll Exp $";
	public static boolean usingZipMaker = false;
	String description;
	String urls[];
	Resource levels[];
	// String secondURL;
	Map prefix2uri; //= new HashMap();
	Map uri2prefix; //= new HashMap();
	public Triples(String us[], String descriptionx, Resource lvls[]) {

		urls = us;
		description = descriptionx;
		levels = lvls;

	}
	Literal jena(ALiteral lit, Model m) {
		if (lit.getDatatypeURI() == null)
			return m.createLiteral(lit.toString(), lit.getLang());
		else
			return m.createTypedLiteral(
				lit.toString(),
				lit.getLang(),
				lit.getDatatypeURI());
	}
	Resource jena(AResource a, Map bnodes, Model m) {
		Resource rslt;
		if (a.isAnonymous()) {
			rslt = (Resource) a.getUserData();
			if (rslt == null) {
				rslt = m.createResource();
				a.setUserData(rslt);
				bnodes.put(rslt, a);
			}
		} else {
			rslt = m.createResource(a.getURI());
		}
		return rslt;
	}
	void initNamespaces() {
		initNamespaces(true);
	}
	void initNamespaces(boolean useNamespaces) {
		prefix2uri = new HashMap();
		uri2prefix = new HashMap();
		if (!useNamespaces)
			return;
		addNamespace("rdfs", "http://www.w3.org/2000/01/rdf-schema#");
		addNamespace("rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
		addNamespace("owl", "http://www.w3.org/2002/07/owl#");
		addNamespace("xsd", "http://www.w3.org/2001/XMLSchema#");
		addNamespace("oiled", "http://oiled.man.example.uk/test#");
		if (urls.length > 0) {
			addNamespace("first", urls[0] + "#");
			if (urls.length > 1) {
				addNamespace("second", urls[urls.length - 1] + "#");
				readNamespaces(1);
			}
			readNamespaces(0);
		}
	}

	abstract InputStream getInputStream(int i);

	public String getTriplesEscaped(int i) {
		return Testy.escape(getTriples(i));
	}

	public String getNamespaces() {
		if (prefix2uri == null)
			initNamespaces();
		String rslt = "";
		Iterator it = prefix2uri.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry pair = (Map.Entry) it.next();
			String p = (String) pair.getKey();
			String uri = (String) pair.getValue();
			if (p.equals("rdf")
				|| p.equals("owl")
				|| p.equals("first")
				|| p.equals("oiled")
				|| p.equals("second")
				|| p.equals("rdfs")
				|| p.equals("xsd"))
				continue;
			rslt += "@prefix " + p + ": &lt;" + uri + "> .\n";
		}
		return rslt;
	}
	void addNamespace(String prefix, String uri) {
		String oldp = (String) uri2prefix.get(uri);
		String oldu = (String) prefix2uri.get(prefix);
		if (oldp != null && !oldp.equals(prefix)) {
			addError(
				"Uri &lt;"
					+ uri
					+ "> is shown using both "
					+ oldp
					+ ": and "
					+ prefix
					+ ":.");
		}
		if (oldu != null && !oldu.equals(uri)) {
			addError(
				"Preifx "
					+ prefix
					+ ": represents both uri &lt;"
					+ uri
					+ "> and &lt;"
					+ oldu
					+ ">.");
		}
		uri2prefix.put(uri, prefix);
		prefix2uri.put(prefix, uri);
	}
	static int plusOne(char a[], int ix) {
		//  System.err.print("<");
		//  System.err.flush();
		try {
			while (true) {
				ix++;
				if (ix >= a.length)
					return -1;
				char ch = a[ix];
				if (ch == '$') {
					do {
						ix++;
					} while (ix < a.length && a[ix] != '$');
					continue;
				}
				if ('a' <= ch && ch <= 'z')
					return ix;
				if ('A' <= ch && ch <= 'Z')
					return ix;
				if ('0' <= ch && ch <= '9')
					return ix;
			}
		} finally {
			// System.err.print(">");
			//  System.err.flush();
		}
	}
	static boolean equalsIgnoringLots(String a, String b) {
		char ac[] = a.toCharArray();
		char bc[] = b.toCharArray();
		int ai = plusOne(ac, -1), bi = plusOne(bc, -1);
		while (true) {
			if (ai == -1 && bi == -1)
				return true;
			if (ai == -1 || bi == -1)
				return false;
			if (Character.toLowerCase(ac[ai]) != Character.toLowerCase(bc[bi]))
				return false;
			ai = plusOne(ac, ai);
			bi = plusOne(bc, bi);
		}

	}
	abstract boolean commentsShouldBeFound();
	/** Creates a new instance of Triples */
	public void readNamespaces(final int wh) {

		// Read the XML to get the namespaces.
		XMLFilter sax =
			new XMLFilterImpl(new org.apache.xerces.parsers.SAXParser()) {
			public void startElement(
				String ns,
				String local,
				String qN,
				Attributes atts) {
				for (int i = 0; i < atts.getLength(); i++) {
					String qn = atts.getQName(i);
					String prefix;
					if (qn.startsWith("xmlns")) {
						if (qn.equals("xmlns")) {
							prefix = "";
						} else if (qn.charAt(5) == ':') {
							prefix = qn.substring(6);
						} else {
							continue;
						}
						addNamespace(prefix, atts.getValue(i));
					}
				}
			}
		};
		try {
			final int cnt[] = new int[] { commentsShouldBeFound() ? 0 : 3 };
			sax.getParent().setFeature(
				"http://xml.org/sax/features/namespaces",
				true);
			sax.getParent().setFeature(
				"http://xml.org/sax/features/namespace-prefixes",
				true);
			sax
				.getParent()
				.setProperty(
					"http://xml.org/sax/properties/lexical-handler",
					new LexicalHandler() {
				private String getFilename() {
					return Triples.this.getFilename(urls[wh]);
				}
				public void endCDATA() {
				}
				public void endDTD() {
				}
				public void startCDATA() {
				}
				public void endEntity(String nm) {
				}
				public void startDTD(String a, String b, String c) {
				}
				public void startEntity(String nm) {
				}
				public void comment(char[] ch, int start, int length) {
					if (ch[0] == '+' && ch[1] == '+')
						return;
					//    System.err.println(new String(ch,start,length));
					cnt[0]++;
					if (cnt[0] > 3) {
						addError(
							(commentsShouldBeFound()
								? "Third and subsequent c"
								: "C")
								+ "omments ignored in "
								+ getFilename()
								+ ".");
					} else {
						String comment = new String(ch, start, length);
						switch (cnt[0]) {
							case 1 :
								if (!equalsIgnoringLots(comment, copyright)) {
									addError(
										"Incorrect copyright statement in "
											+ getFilename()
											+ ".");
								}
								break;
							case 2 :
								if (!equalsIgnoringLots(comment,
									description)) {
									addError(
										"Incorrect description in "
											+ getFilename()
											+ "."
											+ comment
											+ "\n\n"
											+ description);
								}
								break;
							case 3 :
								if (!comment
									.trim()
									.toLowerCase()
									.startsWith("author")) {
									addError(
										"Third comment in "
											+ getFilename()
											+ " is not for the author(s).");
								}
								break;
							default :
								}
					}
				}
			});
			String url = urls[wh];
			InputSource is;

			InputStream in = getInputStream(wh);
			if (in == null) {
				addError("Couldn't open input");
				return;
			}
			is = new InputSource(in);
			is.setSystemId(url);
			sax.parse(is);

			switch (cnt[0]) {
				case 0 :
					addError("No copyright found.");
				case 1 :
					addError("No description found.");
				case 2 :
					addError("Authors not given.");
				default :
					break;
			}
		}
		/*
		catch (RDFException e) {
		    Exception ne =  e.getNestedException();
		    if ( ne != null && ne instanceof SecurityException )
		       throw (RuntimeException)ne;
		    addError(e.getMessage());
		}
		 **/
		catch (SecurityException e) {
			throw e;
		} catch (Exception e) {
			addError(e.getMessage());
		}

	}
	static private boolean isSameURI(String uri1, String uri2) {
		if (uri1 == null || uri2 == null)
			return uri1 == uri2;
		int hash1 = uri1.lastIndexOf('#');
		int hash2 = uri2.lastIndexOf('#');
		return (hash1 == -1 ? uri1 : uri1.substring(0, hash1)).equals(
			hash2 == -1 ? uri2 : uri2.substring(0, hash2));
	}
	public String getTriples(int i) {
		final StringBuffer rslt = new StringBuffer();
		ARP arp = new ARP();
		N3Writer n3w = new N3Writer(rslt);
		arp.setStatementHandler(n3w);
		arp.setErrorHandler(new ErrorHandler() {
			public void error(SAXParseException e) throws SAXException {
				addError(ParseException.formatMessage(e));
			}

			public void fatalError(SAXParseException e) throws SAXException {
				addError(ParseException.formatMessage(e));
				throw e;
			}

			public void warning(SAXParseException e) throws SAXException {
				addError(ParseException.formatMessage(e));

			}
		});
		try {
			String url = urls[i];
			InputStream in = getInputStream(i);
			if (in == null) {
				addError("Couldn't open input");
				return "";
			}
			arp.load(in, url);
		} catch (SecurityException e) {
			throw e;
		} catch (Exception e) {
			addError(e.getMessage());
		}
		if (levels != null) {
			// Do OWL Syntax Check
			try {

				OntDocumentManager dm = new OntDocumentManager();
				dm.setProcessImports(true);
				OntModel m = ModelFactory.createOntologyModel(OWL.NAMESPACE, 
				n3w.m
				, dm);
	
				//Model m = ModelFactory.createDefaultModel();
				//m.write(System.out);
				// m.getDocumentManager();
				Writer w =
					Triples.usingZipMaker
						? CachedURL.errors(urls[i])
						: new StringWriter();
				System.err.print("? " + urls[i] + ".error");
				Checker chk = new Checker(levels[i].equals(OTEST.Lite));
				chk.add(((OntModelImpl)m).getUnionGraph());
				String subLang = chk.getSubLanguage();
				if (!levels[i].getLocalName().equals(subLang)) {
					System.err.println(" not OK " + subLang);
					w.write(
						getFilename(urls[i])
							+ " is in OWL "
							+ subLang
							+ " not "
							+ levels[i].getLocalName()
							+ " <br/>\n");
					if (!(levels[i].equals(OTEST.Full)
						|| subLang.equals("Lite"))) {
						Iterator it = chk.getProblems();
						while (it.hasNext()) {
							SyntaxProblem sp = (SyntaxProblem) it.next();
							sp.setWriter(n3w);
							String s = sp.longDescription();
							s = Util.substituteEntitiesInElementContent(s);
							s = Util.replace(s, "&lt;pre", "<pre");
							s = Util.replace(s, "&lt;/pre", "</pre");
							w.write(s);
							w.write("\n");
						}
					}
					if (!Triples.usingZipMaker)
						addError(((StringWriter) w).toString());
				} else {
					System.err.println(" OK " + subLang);
				}
				w.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}
		return rslt.toString();
	}

	private boolean isKnownURI(String uri) {
		for (int i = 0; i < urls.length; i++)
			if (isSameURI(urls[i], uri))
				return true;
		return false;
	}
	/*
	private void makeDeclaration(String px,String ns) {
	    if (!usingZipMaker)
	        return;
	    if ( done.contains(px) )
	        return;
	    done.add(px);
	}
	 */
	private String uri(String uri) {
		if (!isKnownURI(uri))
			CheckURL.check(Triples.this, uri);
		Iterator it = uri2prefix.keySet().iterator();
		while (it.hasNext()) {
			String ns = (String) it.next();
			if (uri.startsWith(ns)) {
				String prefix = (String) uri2prefix.get(ns);
				//             makeDeclaration();
				return prefix
					+ ":"
					+ escapeURI(uri.substring(ns.length()))
					+ " ";
			}
		}
		if ((!usingZipMaker)
			&& uri.startsWith("http://www.w3.org/2002/03owlt/"))
			uri = uri.substring(30);
		return "<" + escapeURI(uri) + "> ";
	}
	// TODO nodes from imports.
	private String resource(Resource r, Map bnodes) {
		if (r.isAnon()) {
			Object ar = bnodes.get(r);
			if ( ar == null )
			   ar = r.getId().toString();
			String id = ar instanceof AResource ?
			              ((AResource)ar ).getAnonymousID():
			              (String)ar;
			String mapped = (String) bnodes.get(id);
			if (mapped == null) {
				int ii = bnodes.size();
				int nn = ii / 26;
				ii = ii % 26;
				mapped =
					new String(new byte[] {(byte)'?', (byte) (97 + ii)})
						+ (nn > 0 ? ("" + nn) : "");
				bnodes.put(id, mapped);
			}
			return "_:" + mapped + " ";
		} else {
			return uri(r.getURI());
		}
	}
	private String resource(AResource r, Map bnodes) {
		if (r.isAnonymous()) {
			String id = r.getAnonymousID();
			String mapped = (String) bnodes.get(id);
			if (mapped == null) {
				int ii = bnodes.size();
				int nn = ii / 26;
				ii = ii % 26;
				mapped =
					new String(new byte[] {(byte) (97 + ii)})
						+ (nn > 0 ? ("" + nn) : "");
				bnodes.put(id, mapped);
			}
			return "_:" + mapped + " ";
		} else {
			return uri(r.getURI());
		}
	}
	static private String escape(String s, boolean needTripleQuote[]) {
		StringBuffer buf = new StringBuffer();
		char ar[] = s.toCharArray();
		for (int i = 0; i < ar.length; i++) {
			switch (ar[i]) {
				case '\\' :
					buf.append("\\\\");
					break;
				case '\r' :
					buf.append("\\r");
					break;
				case '\t' :
					buf.append("\\t");
					break;
					/**
					case '&' :
					buf.append("&amp;");
					break;
					case '<' :
					buf.append("&lt;");
					break;
					 **/
				case '"' :
                                     if (usingZipMaker) {
					buf.append("\\\"");
					break;
                                     } else {
                                        needTripleQuote[0] = true;
                                     }
				case '\n' :
                                     if (usingZipMaker) {
					buf.append("\\n");
					break;
                                     }else {
                                        needTripleQuote[0] = true;
                                     }
				default :
					if ((!usingZipMaker) || (ar[i] >= 32 && ar[i] <= 126))
						buf.append(ar[i]);
					else {

							buf.append("\\u");
							String hexstr =
								Integer.toHexString(ar[i]).toUpperCase();
							int pad = 4 - hexstr.length();

							for (; pad > 0; pad--)
								buf.append("0");
							buf.append(hexstr);
						}
					
			}
		}
		return buf.toString();
	}


	static private boolean okURIChars[] = new boolean[128];
	static {
		for (int i = 32; i < 127; i++)
			okURIChars[i] = true;
		okURIChars['<'] = false;
		okURIChars['>'] = false;
		okURIChars['\\'] = false;

	}
	static private String escapeURI(String s) {
		StringBuffer buf = new StringBuffer();
		char ar[] = s.toCharArray();
		for (int i = 0; i < ar.length; i++) {
			if (ar[i] < okURIChars.length && okURIChars[ar[i]]) {
				buf.append(ar[i]);
			} else {
				buf.append("\\u");
				String hexstr = Integer.toHexString(ar[i]).toUpperCase();
				int pad = 4 - hexstr.length();

				for (; pad > 0; pad--)
					buf.append("0");
				buf.append(hexstr);
			}
		}
		return buf.toString();
	}
	//abstract int getN();
	private String literal(ALiteral l) {
		StringBuffer buf = new StringBuffer();
                boolean needTripleQuote[] = { false };
                String lexForm = escape(l.toString(),needTripleQuote);
		buf.append("\"");
                if (needTripleQuote[0])
                    buf.append("\"\"");
		buf.append(lexForm);
		buf.append("\"");
                if (needTripleQuote[0])
                    buf.append("\"\"");
		String lang = l.getLang();
		if (lang != null && !lang.equals(""))
			buf.append("@" + lang);
		String dt = l.getDatatypeURI();
		if (dt != null && !dt.equals("")) {
			buf.append("^^" + uri(dt));
		}
		buf.append(" ");
		return buf.toString();
	}
	private String literal(Literal l) {
		StringBuffer buf = new StringBuffer();
                boolean needTripleQuote[] = { false };
                String lexForm = escape(l.toString(),needTripleQuote);
		buf.append("\"");
                if (needTripleQuote[0])
                    buf.append("\"\"");
		buf.append(lexForm);
		buf.append("\"");
                if (needTripleQuote[0])
                    buf.append("\"\"");
		String lang = l.getLanguage();
		if (lang != null && !lang.equals(""))
			buf.append("@" + lang);
		String dt = l.getDatatypeURI();
		if (dt != null && !dt.equals("")) {
			buf.append("^^" + uri(dt));
		}
		buf.append(" ");
		return buf.toString();
	}

}
