/*
 * ZipMaker.java
 *
 * Created on 18 October 2002, 04:50
 */

package wowg;
import java.util.*;
import java.util.zip.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;
import java.io.*;
/**
 *
 * @author  jjc
 */
public class ZipMaker {
    Manifest man;
    ZipOutputStream zip;
    Writer zipw;
    Set dirs;
    /** Creates a new instance of ZipMaker */
    public ZipMaker(String type) {
        man = new Manifest(type,null);
    }
    
    void save()  throws IOException, RDFException {
        Iterator it = man.tests.iterator();
        while ( it.hasNext() ) {
            save((Testx)it.next());
        }
    }
    void createDir(String name)  throws IOException {
        if ( !dirs.contains(name) ) {
        ZipEntry d = new ZipEntry(name);
        zip.putNextEntry(d);
        zip.closeEntry();
        dirs.add(name);
        }
    }
    void addRDFFile(String uri) throws IOException {
        int hash = uri.indexOf("#");
        if ( hash != -1 )
            uri = uri.substring(0,hash);
        
        InputStream in = CachedURL.open(uri);
        
        uri = uri + ".rdf";
        System.err.println(uri);
        String local = uri.substring(ManifestLoader.dir.length()+1);
        int slash = local.indexOf('/');
        if (  slash != -1 )
            createDir(local.substring(0,slash+1));
        zip.putNextEntry(new ZipEntry(local));
        byte buffer[] = new byte[2048];
        int lg;
        do {
            lg = in.read(buffer);
            if ( lg == -1 ) break;
            zip.write(buffer,0,lg);
        } while ( true );
        zip.closeEntry();
    }
    void addFile(String uri, String content) throws IOException {
        String local = uri.substring(ManifestLoader.dir.length()+1);
        int slash = local.indexOf('/');
        if (  slash != -1 )
            createDir(local.substring(0,slash+1));
        zip.putNextEntry(new ZipEntry(local));
        zipw.write(content);
        zipw.flush();
        zip.closeEntry();
    }
    void addFile(String uri) throws IOException {
        addRDFFile(uri);
    }
    void saveN3(Testx t,boolean n3) throws IOException, RDFException {
        String manifest = t.resource.getURI();
        String uris[] = t.getURIs();
        
        
        String desc = t.getDescription();
        WebTriples w3 = WebTriples.get(
          uris,
          desc,
          n3,
		  t.getLevels());
        String heading =
        "#\n" +
        "# Copyright World Wide Web Consortium, (Massachusetts Institute of\n" +
        "# Technology, European Research Consortium for Informatics and\n" +
        "# Mathematics, Keio University).\n"+
        "#\n" +
        "# All Rights Reserved.\n"+
        "#\n" +
        "# Please see the full Copyright clause at\n" +
        "# <http://www.w3.org/Consortium/Legal/copyright-software.html>\n" +
        "#\n";
        StmtIterator si = t.resource.listProperties(DC.creator);
        while ( si.hasNext() ) {
            heading += "# Author: " + si.nextStatement().getString() + "\n";
        }
        heading += "#\n# Description: \n#\n";
        
        StringTokenizer en = new StringTokenizer( desc, "\n" );
        while ( en.hasMoreTokens() ) 
             heading += "# " + en.nextToken() + "\n";
        heading += "#\n";
        for (int i=0; i<uris.length;i++) {
            addFile(uris[i]+(n3?".n3":".nt"),heading+w3.getTriples(i));
        }
       
        
    }
    void save(Testx t)  throws IOException, RDFException{
        String manifest = t.resource.getURI();
        String uris[] = t.getURIs();
        addRDFFile(manifest);
        for (int i=0;i<uris.length;i++)
          addRDFFile(uris[i]);
        saveN3(t,true);
        saveN3(t,false);
    }
    public void saveAs(String fName) throws IOException, RDFException {
        dirs = new HashSet();
        zip = new ZipOutputStream(new FileOutputStream(fName));
        zipw = new OutputStreamWriter(zip,"utf-8");
        save();
        zip.close();
        dirs = null;
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws IOException, RDFException {
        Triples.usingZipMaker = true;
        new ZipMaker("APPROVED").saveAs("approved.zip");
        new ZipMaker("PROPOSED").saveAs("proposed.zip");
    }
    
}
