/*
 * Zipper.java
 *
 * Created on 20 September 2002, 05:54
 */

package wowg;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.mem.*;
import com.hp.hpl.jena.vocabulary.*;

/**
 *
 * @author  jjc
 */
public class Zipper extends HasErrors {
    static public String getParameter(HttpServletRequest req, String nm) {
        String rslt = req.getParameter(nm);
        return rslt==null?"":rslt;
    }
    public String number;
    public String feature;
    public String issue;
    public String description;
    public String first;
    public String second;
    public String approval;
    public String author1; 
    public String author2;
    public String testType;
    public String testDir = null;
    public String firstLevel, secondLevel, entailmentLevel;
    public String size;
    public String start;
    public String section, miscSubSection, dlSubSection, 
         extraSubSection, newSubSection;
    public String oldSubSection;
    //  public String errorMsg = null;
    TestType t;
    /** Creates a new instance of Zipper */
    public Zipper(HttpServletRequest request) {
        
        number = getParameter(request,"number");
        feature = getParameter(request,"feature");
        issue = getParameter(request,"issue");
        description = getParameter(request,"description");
        first = getParameter(request,"first");
        second = getParameter(request,"second");
        author1 = getParameter(request,"author1").trim();
        author2 = getParameter(request,"author2").trim();
        testType = getParameter(request,"testType");
        approval = getParameter(request,"approval");
        firstLevel = getParameter(request,"firstLevel");
        secondLevel = getParameter(request,"secondLevel");
        entailmentLevel = getParameter(request,"entailmentLevel");
        size = getParameter(request,"size");
        section = getParameter(request,"section");
        miscSubSection = getParameter(request,"miscSubSection");
        dlSubSection = getParameter(request,"dlSubSection");
        extraSubSection = getParameter(request,"extraSubSection");
        start = getParameter(request,"start");
        newSubSection = getParameter(request,"newSubSection").trim();
        oldSubSection = "";
        reinit();
    }
    private void reinit() {
        boolean useSections = false;
        testDir = null;
        if ( feature.length() > 0 )
            testDir =  feature +"/";
        else if ( issue.length() > 0 ) {
            int i;
            for (i =1; i<issue.length(); i++)  {
                switch ( issue.charAt(i) ) {
                    case '0': case '1': case '2': case '3': case '4':
                    case '5': case '6': case '7': case '8': case '9':
                    case '.':
                        continue;
                    default:
                        break;
                }
                break;
            }
            testDir =   issue.substring(0,i) +"/";
        } else {
            useSections = true;
            if ( newSubSection.length() > 0 ) {
                miscSubSection = dlSubSection = extraSubSection = "";
            }
            if ( section.length() > 0 ) {
                miscSubSection = section.equals("misc")?miscSubSection:"";
                dlSubSection = section.equals("dl")?dlSubSection:"";
                extraSubSection = section.equals("extra")?extraSubSection:"";
            } else if ( miscSubSection.length() > 0 )
                section = "misc";
            else if ( dlSubSection.length() > 0 )
                section = "dl";
            else if ( extraSubSection.length() > 0 )
                section = "extra";
            
            if ( (!section.equals("")) &&
                  !(newSubSection + miscSubSection+dlSubSection+extraSubSection).equals("") ) {
                      if ( section.equals("misc")) {
                          testDir = "miscellaneous/";
                          oldSubSection = miscSubSection;
                      } else if (section.equals("dl")) {
                          testDir = "description-logic/";
                          oldSubSection = dlSubSection;
                      } else if ( section.equals("extra")) {
                          testDir = "extra-credit/";
                          oldSubSection = extraSubSection;
                      }
                      if ( newSubSection.length() > 0 ) {
                          oldSubSection = section + "-" + start + "-" + newSubSection;
                      } else {
                          int firstMinus = oldSubSection.indexOf('-');
                          int secondMinus = oldSubSection.indexOf('-',firstMinus+1);
                          start = oldSubSection.substring(firstMinus+1,secondMinus);
                      }
                      if ( number.length() == 2 )
                          number = start.substring(0,1) + number;
            }
        }
        if (!useSections) {
            section = newSubSection =
                miscSubSection = dlSubSection = extraSubSection = "";
        }
        t = TestType.lookup(testType);
        approval = approval.trim();
        if ( approval.length() > 0 ) {
            int hash = approval.lastIndexOf('#');
            if ( hash == -1 ) hash = approval.length();
            int dot = approval.lastIndexOf('.',hash);
            int slash = approval.lastIndexOf('/',hash);
            if ( dot > slash ) {
                addError("Removing suffix from approval URI.");
                approval = approval.substring(0,dot) + approval.substring(hash);
            }
        }
        if ( number.length()==2 )
            number = "0" + number;
    }
    Resource r;
    void read(Model m, String s) throws RDFException {
        try {
        InputStream in = CachedURL.open(s);
        m.read(in,s);
        in.close();
        }
        catch (IOException e) {
            throw new RDFException(e);
        }
    }
    public void refresh()  {
        String urls = url("Manifest");
        Model m = new ModelMem();
        String test;
        try {
            try {
                read(m,urls);
                test = urls + "#test";
                r = m.createResource(test);
            }
            catch (RDFException firstE) {
                addError("Manifest not found.");
                return;
            }
            if ( !r.listProperties().hasNext() ) {
                addError("Could not find test details in " + getFilename(urls) +".");
                return;
            }
            Testx testx = Testx.make(r);
            if ( testx == null ) {
                addError("Test " + number + " not found in " + getFilename(urls) +".");
                return;
            }
            // testx.setEscape(false);
            testType = testx.getTestTypeFrag();
            author2 = testx.getProperty(DC.creator);
            author1 = testx.getProperty(DC.creator);
            // First
            first = testx.getFile(0);
            second = testx.getFile(1);
            firstLevel = testx.getLevel(0);
            secondLevel = testx.getLevel(1);
            entailmentLevel = testx.getLevel();
            size = OTEST.Small.equals(testx.getSize()) ? "Small" :
                   OTEST.Large.equals(testx.getSize()) ? "Large" :
                     "";
//            System.err.println("Has Property = " +
//            testx.resource.hasProperty(OTEST.size) ); 
            if ( firstLevel == null )
                firstLevel = "";
            if ( secondLevel == null )
                secondLevel = "";
            if ( entailmentLevel == null )
                entailmentLevel = "";
            description = testx.getDescription();
            if ( r.hasProperty(RTEST.approval) )
                approval = testx.uriOf(RTEST.approval);
            else
                approval = "";
            Resource is = testx.getIssue();
            if ( is != null )
                issue = getFragment(is.getURI());
            else
                issue = "";
            
            start = section = miscSubSection =
            dlSubSection = extraSubSection = newSubSection =
             oldSubSection = "";
            
            is = testx.getIssuette();
            if ( is != null ) {
                String uri = is.getURI();
                oldSubSection = uri.substring(31);
                int firstDash = oldSubSection.indexOf('-');
                int secondDash = oldSubSection.indexOf('-',firstDash+1);
                section = oldSubSection.substring(0,firstDash);
                start = oldSubSection.substring(firstDash+1,secondDash);
                if ( section.equals("misc") )
                    miscSubSection = oldSubSection;
                else if ( section.equals("dl") )
                    dlSubSection = oldSubSection;
                else 
                    extraSubSection = oldSubSection;
            }
                
            mergeErrors(testx);
        }
        catch (Exception re) {
            re.printStackTrace();
            addError(re);
        }
        reinit();
    }
    
    private String url(String part) {
        if ( part == null )
            return null;
        if ( testDir == null ) {
            addError( "You should specify a feature or an issue." );
            testDir = "miscellaneous/";
        }
        return ManifestLoader.dir + "/" + testDir + part + number;
    }
    private String[] urls(String a,String b) {
        String rslt[] = new String[b==null?1:2];
        rslt[0] = url(a);
        if ( b != null )
            rslt[1] = url(b);
        return rslt;
    }
    private String level(String s,String s2) {
        return "<option " + ( s.equals(s2)?"selected":"" )+ ">"+s2+"</option>\n";
    }
    public String levels(String s) {
        return level(s,"") + level(s,"Lite") + level(s,"DL") + level(s,"Full");
    }
    
    public String sizes(String s) {
        return level(s,"") + level(s,"Small") + level(s,"Large");
    }
    public void boiler() {
        
        String uri1 =  url(t.getFirst());
        approval = "";
        
        first =
        "<rdf:RDF\n" +
        "    xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" +
        "    xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\"\n" +
        "    xmlns:owl=\"http://www.w3.org/2002/07/owl#\"\n" +
        "    xmlns:first=\"" + uri1 +"#\"\n" ;
        if ( t.getSecond() == null ) {
            first = first +
            "    xml:base=\"" + uri1 + "\" " + ">\n\n\n" +
            "</rdf:RDF>";
            second = "";
        } else {
            String uri2 = url(t.getSecond());
            first = first +
            "    xmlns:second=\"" + uri2 +"#\"\n" ;
            second = first;
            first = first +
            "    xml:base=\"" + uri1 + "\" " + ">\n\n\n" +
            "</rdf:RDF>";
            second = second +
            "    xml:base=\"" + uri2 + "\" " + ">\n\n\n" +
            "</rdf:RDF>";
        }
    }
    
    private ZipOutputStream zip;
    private PrintWriter pw;
    public void zip(OutputStream out) throws IOException {
        zip = new ZipOutputStream(out);
        pw = new PrintWriter(new OutputStreamWriter(zip,"utf-8"));
        zip.putNextEntry(new ZipEntry(testDir));
        zip.closeEntry();
        startZipEntry("Manifest");
        pw.println("<rdf:RDF xmlns:rdf='http://www.w3.org/1999/02/22-rdf-syntax-ns#'");
        pw.println("      xmlns:rdfs='http://www.w3.org/2000/01/rdf-schema#'");
        pw.println("      xmlns:rtest='http://www.w3.org/2000/10/rdf-tests/rdfcore/testSchema#'");
        pw.println("      xmlns:otest='http://www.w3.org/2002/03owlt/testOntology#'");
        pw.println("      xmlns:dc='http://purl.org/dc/elements/1.0/'");
        pw.println("      xmlns='http://www.w3.org/1999/xhtml'");
        pw.println("      xmlns:owl='http://www.w3.org/2002/07/owl#'");
        pw.println("      xml:base='" + url("Manifest")+"'");
        pw.println(">");
        pw.println("<otest:" + t.getName() + " rdf:ID='test'>");
        pw.println("   <rtest:status>" + ( approval.length()>0 ? "APPROVED" : "PROPOSED") + "</rtest:status>");
        if ( approval.length() > 0 )
            pw.println("   <rtest:approval rdf:resource='" + approval + "' />");
        if ( author1.length() > 0 )
            pw.println("   <dc:creator>"+author1+"</dc:creator>");
        if ( author2.length() > 0 && (!author1.equals(author2)) )
            pw.println("   <dc:creator>"+author2+"</dc:creator>");
        if ( issue.length() > 0 )
            pw.println("   <rtest:issue rdf:resource='http://www.w3.org/2001/sw/WebOnt/webont-issues#" +
            issue+ "' />");
        if ( feature.length() > 0 )
            pw.println("   <otest:feature rdf:resource='http://www.w3.org/2002/07/owl#" +
            feature+ "' />");
        if ( oldSubSection.length() > 0 )
            pw.println("   <otest:issuette rdf:resource='http://www.w3.org/TR/owl-test/#" +
            oldSubSection + "' />");
        if ( entailmentLevel.length() > 0 )
            pw.println("   <otest:level rdf:resource='http://www.w3.org/2002/03owlt/testOntology#" +
            entailmentLevel+ "' />");
        if ( size.length() > 0 )
            pw.println("   <otest:size rdf:resource='http://www.w3.org/2002/03owlt/testOntology#" +
            size+ "' />");
        pw.println("   <rtest:description rdf:parseType='Literal'>\n"+description+"</rtest:description>");
        if ( t.getSecond() == null ) {
            String firstFileProp;
            if ( t.getName().equals("TrueTest") || t.getName().equals("OWLforOWLTest") ) {
                firstFileProp = "rtest:conclusionDocument";
            } else {
                firstFileProp = "rtest:inputDocument";
            }
            pw.println("   <"+firstFileProp+">");
            pw.println("         <rtest:RDF-XML-Document rdf:about='" + t.getFirst() + number + "' >");
            if ( firstLevel.length() > 0 ) {
            pw.println("              <otest:level rdf:resource='http://www.w3.org/2002/03owlt/testOntology#" +
            firstLevel+ "' />");
            }
            pw.println("         </rtest:RDF-XML-Document>");
            pw.println("   </"+firstFileProp+">");
        } else {
            String firstFileProp, secondFileProp;
            if ( t.getName().equals("ImportsLevelTest") ) {
                firstFileProp = "otest:importedPremiseDocument";
                secondFileProp = "rtest:inputDocument";
            } else {
                firstFileProp = "rtest:premiseDocument";
                secondFileProp = "rtest:conclusionDocument";
            }
            pw.println("   <"+firstFileProp+">");
            pw.println("         <rtest:RDF-XML-Document rdf:about='" + t.getFirst() + number + "' >");
            if ( firstLevel.length() > 0 ) {
            pw.println("              <otest:level rdf:resource='http://www.w3.org/2002/03owlt/testOntology#" +
            firstLevel+ "' />");
            }
            pw.println("         </rtest:RDF-XML-Document>");
            pw.println("   </"+firstFileProp+">");
            pw.println("   <"+secondFileProp+">");
            pw.println("         <rtest:RDF-XML-Document rdf:about='" + t.getSecond() + number + "' >");
            if ( secondLevel.length() > 0 ) {
            pw.println("              <otest:level rdf:resource='http://www.w3.org/2002/03owlt/testOntology#" +
            secondLevel+ "' />");
            }
            pw.println("         </rtest:RDF-XML-Document>");
            pw.println("   </"+secondFileProp+">");
        }
        pw.println("</otest:" + t.getName() + " >");
        pw.println("</rdf:RDF>");
        pw.flush();
        zip.closeEntry();
        addZipEntry(t.getFirst(),first);
        if ( t.getSecond() != null )
            addZipEntry(t.getSecond(),second);
        pw.close();
        zip.close();
        out.close();
    }
    private void startZipEntry(String fname) throws IOException {
        zip.putNextEntry(new ZipEntry(testDir + fname+number+".rdf"));
        pw.println("<!--++ " + testDir + fname+number+".rdf ** generated using webont test editor. ++-->");
        pw.println("<!--++ Created " + new Date().toGMTString() + " ++-->");
        pw.println("<!--");
        pw.println("  Copyright World Wide Web Consortium, (Massachusetts Institute of");
        pw.println("  Technology, European Research Consortium for Informatics and");
        pw.println("  Mathematics, Keio University).");
        pw.println("");
        pw.println("  All Rights Reserved.");
        pw.println("");
        pw.println("  Please see the full Copyright clause at");
        pw.println("  <http://www.w3.org/Consortium/Legal/copyright-software.html>");
        pw.println("");
        pw.print("  $");
        pw.println("Id: This string is updated by cvs. $");
        pw.println("-->");
    }
    private void addZipEntry(String fname, String rdfxml ) throws IOException {
        startZipEntry(fname);
        pw.println("<!--");
        pw.println(description);
        pw.println("-->");
        if ( author1.length() > 0 )
            pw.println("<!-- Author: " + author1 + " -->");
        else if ( author2.length() > 0 )
            pw.println("<!-- Author: " + author2 + " -->");
        pw.println(rdfxml);
        pw.flush();
        zip.closeEntry();
    }
    
    public Testy validate() {
        return new Testy() {
            public int getN() {
                return t.getSecond()==null?1:2;
            }
            public String getTestType() {
                return t.getName();
            }
            public String getName() {
                return number;
            }
            public String getDescription() {
                return description;
            }
            public String getTestURI() {
                return url("Manifest")+"#test";
            }
            public String getURI(int i) {
                return url(i==0?t.getFirst():t.getSecond());
            }
            public String getAuthor() {
                if ( author1.equals("") ) return author2;
                return author1;
            }
            Resource str2level(String s) {
                if ( s == null )
                    return null;
                if ( s.equals("Lite") ) 
                    return OTEST.Lite;
                if ( s.equals("DL") ) 
                    return OTEST.DL;
                if ( s.equals("Full") )
                    return OTEST.Full;
                return null;
            }
                    Resource getLevelR() {
                        
                        return str2level(entailmentLevel);
                    }
                    Resource getLevelR(int i) {
                        return str2level(i==0?firstLevel:secondLevel);
                    }
            public Triples asTriples() {
                
                return new Triples(  urls(t.getFirst(),t.getSecond()), description,
                  new Resource[] {
                  	getLevelR(0),
                  	getLevelR(1)
                  } ) {
                    boolean commentsShouldBeFound() {
                        return false;
                    }
                    InputStream getInputStream(int i) {
                        return i==0 ?
                          getFirstInputStream():
                              getSecondInputStream();
                    }
                    InputStream getFirstInputStream() {
                        try  {
                            return new ByteArrayInputStream(first.getBytes("ASCII"));
                        }
                        catch (UnsupportedEncodingException e) {
                            System.err.println(e.getMessage());
                            return null;
                        }
                    }
                    InputStream getSecondInputStream(){
                        try  {
                            return new ByteArrayInputStream(second.getBytes("ASCII"));
                        }
                        catch (UnsupportedEncodingException e) {
                            System.err.println(e.getMessage());
                            return null;
                        }
                    }
                };
            }
            public String getFileHeading(int i) {
                return i==0?t.getFirst():t.getSecond();
            }
            public String getFile(int i) {
                return i==0?first:second;
            }
            public Resource getSize() {
                return null;
            }
        };
    }
}
