<%@page %>
<%-- This file is the text of the working draft. 
 It excludes the stuff before the TOC; the TOC; and the tests themselves.
--%>
<%=head(1,"introduction","Introduction")%>

<p>
As part of the definition of the Web Ontology Language (OWL)
the Web Ontology Working Group provides  <em>a
set of  <strong>test cases</strong></em>. This document 
presents those test cases.
They are intended to provide examples for, and clarification
of, the normative definition of OWL
 found in 
<%=ref("OWL Semantics and Abstract Syntax")%>
to which this document is subsiduary.
</p>
<p>
This document describes the various types of test used
and the format in which the tests
are presented. 
Alternative formats of the test collection are provided.
These are intended to be suitable
for use by OWL developers in test harnesses,
possibly as part of a test driven development process,
such as Extreme Programming <%=ref("XP")%>.
The format of the <code>Manifest</code> files
used as part of these alternative formats is described.
</p>
<p>
This document describes the process for conflict 
resolution and errata related to these tests.
</p>
<p>In the non-normative appendices, this document also
describes the 
process for creation and approval of these tests.
</p>
<p>
Further appendices show further proposed tests that are awaiting
resolution by the working group.
</p>
<%=head(2,"scope","Conformance and Scope")%>
<p>
Various conformance levels are defined in <a href="#conformance">this document</a> 
in terms
of
<%=ref("OWL Semantics and Abstract Syntax")%>.
</p>
<p>
However, the test cases do <em>not</em> constitute a conformance test suite for OWL.
</p>
<p>
The tests illustrate issue resolutions, and illustrate the use and meaning
of
the terms in the OWL namespace.
</p>
<p>
There are 
other miscellaneous tests:
 some
arising in the literature, and in preexisting systems;
 others intending to show the difficulty of
complete implementations of OWL Full.

</p>

<%=head(1,"deliverables","Deliverables (Normative)")%>
<p>
The deliverables included as part of the test cases are:
</p>
<ul>
<li><%=size==1?"This":"The <a href='./'>"%> recommendation track document<%=size==1?"":"</a>"%>, which normatively includes the tests, either explicitly or, 
for the larger tests, by reference to the test Web site.</li>
<li>Additional alternative forms of the normative tests. These are also
included in informative appendices to this document.</li>
<li>A zip file including 
<ul>
<li>The RDF/XML files that participate in the
tests </li>
<li>Manifest files indicating which tests should be applied to which
test files.</li>
<li>The non-normative alternative forms of the test files.</li>
</ul>
This zip file is a non-normative alternative format.
</li>
<li>
A
<a href="Manifest.rdf">Manifest</a>
file describing all the tests.
</li>
<li>A 
<a href="http://www.w3.org/2002/03owlt/">web-site</a> 
which includes the files in the zip file.
This also is a non-normative alternative format.</li>
</ul>
<p class="note">
<strong>Note:</strong> Other files can be found
under the top URL of the web-site which are not part
of the deliverable.
</p>
<p class="todo">
[[EDITORS' NOTE: Do we need index files for the web site that clarify which parts are part of 
the deliverable and which are not?]]
</p>
<%=head(2,"normativity","Normative Status")%>
<p>
Of the deliverables the only normative tests are 
<a href="#approved">those</a> included
in this document. All other deliverables,
are informative. Moreover, the recommendation document is
informative except for the conformance statements, the
test data (specified in RDF/XML 
<%=ref("RDF/XML Syntax")%>), and the supporting documentation.
</p>
<%--
<%=head(2,"conflict","Conflict Resolution")%>
<p>
When the normative tests and the other normative
OWL recommendations diverge the following process should be followed.
</p>
<p>If the OWL working drafts are at or before Proposed Recommendation then:
</p>
<ol>
<li>The conflict is reported to 
<a href="mailto:public-webont-comments@w3.org">public-webont-comments@w3.org</a>.
</li>
<li>The working group 
makes appropriate modifications to remove the conflict.</li>
</ol>
<p>If the OWL recommendation has passed Proposed Recommendation then:
</p>
<ol>
<li>The conflict is reported to 
<a href="mailto:public-webont-comments@w3.org">public-webont-comments@w3.org</a>.</li>
<li>The working group, or its successors, considers the conflict.
<br />
<br />
While this happens
the other recommendation documents take precedence over the test case.
</li>
</ol>
<ol>
<li>If there is working group consensus to retain the test case as normative
and to publish an erratum against the other recommendation document(s)
then this is done.</li>
<li>Otherwise an erratum is published which deletes the test case.
</li>
</ol>
<p class="note">
<strong>Note:</strong> this last step does not require consensus, or even a majority.
</p>
<p class="note">
<strong>Note:</strong> the errata process over the tests is monotonic decreasing;
i.e. after Proposed Recommendation, any system that passes all the test cases
will necessarily pass all the test case with errata.
</p>
--%>
<%=head(1,"testTypes","Test Types (Normative)")%>
<p>
Each test consists of either one or more RDF/XML documents and a <code>Manifest</code>
file.
Tests of one document indicate some property of that document
when viewed as an OWL knowledge base.
Tests of two or more documents indicate a relationship between the two documents
when viewed as OWL knowledge bases.
</p>
<p>
The <code>Manifest</code>
file is named <%=named("Manifest")%> (The <code><em>NNN</em></code>
is replaced by the test number).
It contains metadata (in RDF) indicating the test type,
and describing the test.
</p>
<%=head(2,"testBadOwl","Tests for Incorrect Use of OWL Namespace")%>
<p>
These tests use one document.
It is named <%=named("bad")%>.
This document includes a use of the OWL namespace with a local name 
that is not defined by the OWL recommendation. An OWL Syntax checker SHOULD
give a warning.
</p>
<p class="note">
<strong>Note:</strong> These tests are intended to help migration 
from DAML+OIL <%=ref("DAML+OIL")%>,
since the local names chosen are defined in the DAML+OIL namespace.
</p>
<%=head(2,"testEntailment","Entailment Tests")%>
<p>
These tests use two documents.
One is named <%=named("premises")%>, 
the other is named <%=named("conclusions")%>.
The <code>conclusions</code> are 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#direct_entails">entailed</a> by the <code>premises</code>.
Such entailment is defined by the OWL semantics <%=ref("OWL Semantics and Abstract Syntax")%>,
(see also 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#rdfs_entails_Full"
>OWL Full entailment</a
>).
</p>
<%=head(2,"testNonEntailment","Non-Entailment Tests")%>
<p>
These tests use two documents.
One is named <%=named("premises")%>, 
the other is named <%=named("nonconclusions")%>.
The <code>nonconclusions</code> are not 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#direct_entails">entailed</a>
 by the <code>premises</code>.
Such entailment is defined by the OWL semantics <%=ref("OWL Semantics and Abstract Syntax")%>,
(see also 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#rdfs_entails_Full"
>OWL Full entailment</a
>).</p>
<%=head(2,"testTrue","True Tests")%>
<p>
These tests use one document.
It is named <%=named("conclusions")%>.
The <code>conclusions</code> follow from the OWL semantics 
<%=ref("OWL Semantics and Abstract Syntax")%>.
These tests are a special case of the <a href="#testEntailment">entailment tests</a>
in which the premises are empty.</p>
<%=head(2,"testOWLforOWL","OWL for OWL Tests")%>
<p>
These tests use one document.
It is named <%=named("conclusions")%>.
These are a special case of <a href="#testTrue">true tests</a>.
The <code>conclusions</code> follow from the 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#5.3"
>OWL Full semantics</a
>
<%=ref("OWL Semantics and Abstract Syntax")%>.
The tests are intended to illustrate how 
OWL Full can be used to describe its own properties and
classes.</p>
<%=head(2,"testConsistency","Consistency Tests")%>
<p>
These tests use one document. 
It is named <%=named("consistent")%>.
The document is 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#direct_consistent">consistent</a>
as defined
by the  OWL Semantics <%=ref("OWL Semantics and Abstract Syntax")%>,
(see also 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#rdfs_consistent_Full"
>OWL Full consistency</a
>).
</p>
<%=head(2,"testInconsistency","Inconsistency Tests")%>
<p>
These tests use one document. 
It is named <%=named("inconsistent")%>.
The document is not
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#direct_consistent">consistent</a>
as defined
by the  OWL semantics <%=ref("OWL Semantics and Abstract Syntax")%>,
(see also 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#rdfs_consistent_Full"
>OWL Full consistency</a
>).
</p>
<%=head(2,"testImportEntailment","Import Entailment Tests")%>
<p>
These tests use more than two documents.
One is named <%=named("premises")%>, 
another is named <%=named("conclusions")%>, the rest have names beginning <%=named("support")%>.
The <code>support</code> documents are in the <a href=
"http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#RDF_graph_imports_closure">
imports closure</a>  of the
 <code>premises</code> document.
The <code>conclusions</code> are 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#direct_entails">entailed</a>
 by the <a href=
"http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#RDF_graph_imports_closure">
imports closure</a>
of the <code>premises</code>.
Such entailment is defined by the OWL semantics <%=ref("OWL Semantics and Abstract Syntax")%>,
(see also 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#rdfs_entails_Full"
>OWL Full entailment</a
>).</p>
<%=head(2,"testImportLevel","Import Level Tests")%>
<p>
These tests use two documents.
One is named <%=named("imports")%>, 
the other is named <%=named("main")%>.
These
 tests indicate the 
interaction between <code>owl:imports</code>
and the sublanguage levels of the <code>main</code> document.</p>

<%=head(1,"conformance","Conformance (Normative)")%>
<%=head(2,"docConformance","Document Conformance")%>

<%=head(3,"syntaxConformance","Syntactic Conformance")%>
<p>
An <a id="dfn-OWL-Full-document"  name="dfn-OWL-Full-document"><dfn class="conformance">OWL Full
document</dfn></a>
is any
<a href="http://www.w3.org/2001/sw/RDFCore/TR/WD-rdf-syntax-grammar-20030117/">
RDF/XML document</a> <%=ref("RDF/XML Syntax")%>.
<!--
for which the corresponding
<a href="http://www.w3.org/2001/sw/RDFCore/TR/WD-rdf-concepts-20030117/#dfn-rdf-graph">
RDF graph
</a>
<%=ref("RDF Concepts")%>
does not use any URI references 
starting with the prefix <code>http://www.w3.org/2002/07/owl#</code>
except those found in the
<%=ref("RDF Schema for OWL")%>.
-->
</p>
<p>
An <a id="dfn-OWL-DL-document"  name="dfn-OWL-DL-document"><dfn class="conformance">OWL DL
document</dfn></a> is an <a href="#dfn-OWL-Full-document">OWL Full document</a>
such that the 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#RDF_graph_imports_closure">imports closure</a>
<%=ref("OWL Semantics and Abstract Syntax")%>
of the corresponding
<a href="http://www.w3.org/2001/sw/RDFCore/TR/WD-rdf-concepts-20030117/#dfn-rdf-graph">
RDF graph
</a>
<%=ref("RDF Concepts")%>
is an 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/mapping.html#OWL_DL_RDF_graphs">OWL DL ontology in RDF
graph form</a>.
</p>
<p>
An <a id="dfn-OWL-Lite-document" name="dfn-OWL-Lite-document"><dfn class="conformance">OWL Lite
document</dfn></a> is an <a href="#dfn-OWL-Full-document">OWL Full document</a>
such that the 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#RDF_graph_imports_closure">imports closure</a>
<%=ref("OWL Semantics and Abstract Syntax")%>
of the corresponding
<a href="http://www.w3.org/2001/sw/RDFCore/TR/WD-rdf-concepts-20030117/#dfn-rdf-graph">
RDF graph
</a>
<%=ref("RDF Concepts")%>
is an 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/mapping.html#OWL_Lite_RDF_graphs">OWL Lite ontology in RDF
graph form</a>.
</p>

<%=head(3,"consistencyConformance","Semantic Conformance")%>


<p>
An OWL document is  
<a id="dfn-consistent-OWL-document" name="dfn-consistent-OWL-document"><dfn class="conformance">consistent</dfn></a> 
if and only if
 there exists some <em>model</em>
 of the document that is consistent with the
 constraints specified by the
relevant <em>model theory</em> 
(see <%=ref("OWL Semantics and Abstract Syntax")%>:
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#direct_consistent">
OWL Lite and OWL DL</a>, 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/rdfs.html#rdfs_consistent_Full">
OWL 
Full</a>).
</p>


<%=head(2,"checkerConformance","Document Checker Conformance")%>
<p>
This section uses the words MUST, MUST NOT, SHOULD and MAY as in <%=ref("RFC 2119")%>.
</p>

<%=head(3,"syntaxChecker","Syntax Checker")%>

<p>
An <a name="dfn-OWL-syntax-checker" id="dfn-OWL-syntax-checker"><dfn class="conformance">OWL
syntax checker</dfn></a> 
takes a document as input, and outputs one word being one of <code>Lite</code>,
<code>DL</code>, <code>Full</code>, <code>Other</code>.
</p>
<p>
The output MUST conform with the following:
</p>
<dl>
<dt>Lite</dt>
<dd>The input document is an <a href="#dfn-OWL-Lite-document">OWL Lite document</a>.</dd>
<dt>DL</dt>
<dd>The input document is an <a href="#dfn-OWL-DL-document">OWL DL document</a>
 but not an <a href="#dfn-OWL-Lite-document">OWL Lite document</a>.</dd>
<dt>Full</dt>

<dd>The input document is an <a href="#dfn-OWL-Full-document">OWL Full document</a>
 but not an <a href="#dfn-OWL-DL-document">OWL DL document</a>.</dd>
<dt>Other</dt>
<dd>The input document is not an <a href="#dfn-OWL-Full-document">OWL Full document</a>.</dd>
</dl>
<p>
In addition, an OWL Syntax Checker SHOULD give a warning if
 the 
<a href="http://www.w3.org/2001/sw/RDFCore/TR/WD-rdf-concepts-20030117/#dfn-rdf-graph">
RDF graph
</a>
<%=ref("RDF Concepts")%>
corresponding to the document
uses any URI references 
starting with the prefix <code>http://www.w3.org/2002/07/owl#</code>
except those found in the
<%=ref("RDF Schema for OWL")%>.
</p>
<p>
<a name="network_errors" id="network_errors"> </a>
An <a href="#dfn-OWL-syntax-checker">OWL
syntax checker</a> SHOULD report network errors occurring
during the computation of the imports closure.
</p>
<%=head(3,"consistencyChecker","Consistency Checker")%>
<p>
An <a name="dfn-OWL-consistency-checker" id="dfn-OWL-consistency-checker"><em>OWL consistency checker</em></a>
takes a document as input, and outputs one word being <code>Consistent</code>,
<code>Inconsistent</code>, or <code>Unknown</code>.
</p>
<p>
An <a href="#dfn-OWL-consistency-checker">OWL
consistency checker</a> SHOULD report network errors occurring
during the computation of the imports closure.
</p>
<p>
An OWL consistency checker MUST return <code>Consistent</code> only when the
input document is consistent and <code>Inconsistent</code> only when the input
document is not consistent (this property is usually called
<em>soundness</em>).
</p>
<p>
An OWL consistency checker is 
<a name="dfn-complete" id="dfn-complete"><em>complete</em></a>, with respect to 
datatypes supported by its
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#datatype_theory">
datatype theory</a> 
<%=ref("OWL Semantics and Abstract Syntax")%>, if, given sufficient (but
finite) resources (CPU cycles and memory)
and the absence of 
<a href="#network_errors">network errors</a>, it will always return
either <code>Consistent</code> or <code>Inconsistent</code>; 
otherwise it is 
<a name="dfn-incomplete" id="dfn-incomplete"><em>complete</em></a>. It has
been shown that for OWL Lite and DL it is possible to construct a
<a href="#dfn-complete">complete</a> consistency checker (the languages are <em>decidable</em>), 
and that
for OWL full it is not possible to construct a <a href="#dfn-complete">complete</a>
consistency
checker (the language is <em>undecidable</em>).
</p>
<p>
The 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#datatype_theory">
datatype theory</a> of
an OWL consistency checker SHOULD minimally support at least 
<a href=
"http://www.w3.org/TR/2001/REC-xmlschema-2-20010502/#integer"><code>xsd:integer</code></a>, <a href=
"http://www.w3.org/TR/2001/REC-xmlschema-2-20010502/#string"><code>xsd:string</code></a>
from <%=ref("XML Schema Datatypes")%>.
</p>
<p>
Five different conformance classes of 
<a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> are defined.
</p>
<!--
<p>
An <a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> MUST NOT output <code>Consistent</code>
unless the input document is a <a href="#dfn-consistent-OWL-document">
consistent OWL document</a>.
</p>
<p>
An <a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> MUST NOT output <code>Inconsistent</code>
unless the input document is not a <a href="#dfn-consistent-OWL-document">
consistent OWL document</a>.
</p>
<p>
An <a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> SHOULD report 
network errors occurring
during the computation of the imports closure.
</p>
-->
<p>
An <a name="dfn-OWL-Lite-consistency-checker"><dfn class="conformance">OWL Lite consistency checker</dfn></a>
is an <a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> that
takes an <a href="#dfn-OWL-Lite-document">OWL Lite document</a> as input.
</p>
<p>
An <a name="dfn-OWL-DL-consistency-checker"><dfn class="conformance">OWL DL consistency checker</dfn></a>
is an <a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> that
takes an <a href="#dfn-OWL-DL-document">OWL DL document</a> as input.
</p>
<p>
An <a name="dfn-OWL-Full-consistency-checker"><dfn class="conformance">OWL Full consistency checker</dfn></a>
is an <a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> that
takes an <a href="#dfn-OWL-Full-document">OWL Full document</a> as input.
</p>
<div class="note">
<p ><strong>Note:</strong>
Every  <a href="#dfn-OWL-Full-consistency-checker">OWL Full consistency checker</a> 
is also an <a href="#dfn-OWL-DL-consistency-checker">OWL DL consistency checker</a>.
Every  <a href="#dfn-OWL-DL-consistency-checker">OWL DL consistency checker</a> 
is also an <a href="#dfn-OWL-Lite-consistency-checker">OWL Lite consistency checker</a>.
Every  <a href="#dfn-OWL-Lite-consistency-checker">OWL Lite consistency checker</a>
 can be trivially transformed into an <a href="#dfn-OWL-Full-consistency-checker">OWL Full consistency checker</a>.
</p>
<p>The different levels are intended to be used to indicate the intended domain of
a consistency checker.</p>
<p><strong>Note:</strong> there are trivial implementations of these consistency
checkers; for example, one which always outputs <code>Unknown</code>.
</p>
</div>
<p>
A <a name="dfn-complete-OWL-Lite-consistency-checker"><dfn class="conformance">complete OWL Lite consistency checker</dfn></a>
is an <a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> that
is <a href="#dfn-complete">complete</a>.
</p>
<p>
A <a name="dfn-complete-OWL-DL-consistency-checker"><dfn class="conformance">complete OWL DL consistency checker</dfn></a>
is an <a href="#dfn-OWL-consistency-checker">OWL consistency checker</a> that
is <a href="#dfn-complete">complete</a>.
</p>

<div class="note">
<p ><strong>Note:</strong>
<a href="#dfn-complete-OWL-DL-consistency-checker"><dfn class="conformance">Complete OWL DL consistency checkers</dfn></a>
and 
<a href="#dfn-complete-OWL-Lite-consistency-checker"><dfn class="conformance">Complete OWL Lite consistency checkers</dfn></a>
MAY return <code>Unknown</code> on an <a href="#dfn-OWL-DL-document">OWL DL document</a>
or <a href="#dfn-OWL-Lite-document">OWL Lite document</a> in the case where
a resource limit has been exceeded.
</p>
</div>



<%=head(1,"running","Testing an OWL Implementation (Informative)")%>
<%--
<p class="todo">[WG note:
The use of must, should etc. is consistent with <%=ref("RFC 2119")%>,
but since this section is informative I have not imported the definitions.]
</p>
--%>
<%=head(2,"runningSyntaxChecker","OWL Syntax Checkers")%>
<p>
An <a href="#dfn-OWL-syntax-checker">OWL syntax checker</a> 
when presented with any of the test files
must return the indicated result.
</p>
<%=head(2,"runningConsistencyChecker","OWL Consistency Checker")%>
<p>
An <a href="#dfn-OWL-Lite-consistency-checker">OWL Lite consistency checker</a>, 
when presented with a file from
an OWL Lite <a href="#testConsistency">consistency test</a>, 
must output <code>Consistent</code>
or <code>Unknown</code>.
</p>
<p>
An <a href="#dfn-OWL-DL-consistency-checker">OWL DL consistency checker</a>, 
when presented with a file from
an OWL DL or OWL Lite <a href="#testConsistency">consistency test</a>, 
must output <code>Consistent</code>
or <code>Unknown</code>.
</p>
<p>
An <a href="#dfn-OWL-Full-consistency-checker">OWL Full consistency checker</a>, 
when presented with a file from
an OWL Full, OWL DL or OWL Lite <a href="#testConsistency">consistency test</a>, 
must output <code>Consistent</code>
or <code>Unknown</code>.
</p>
<p>
The corresponding <a href="#testInconsistency">inconsistency tests</a> must result in output of
<code>Inconsistent</code> or <code>Unknown</code>, as long as the datatypes required
by the test are supported by the 
<a href="http://www.w3.org/TR/2003/WD-owl-semantics-20030331/direct.html#datatype_theory">
datatype theory</a> of the consistency checker.
</p>
<p>
A <a href="#dfn-complete-OWL-Lite-consistency-checker">complete OWL Lite consistency checker</a> or a 
<a href="#dfn-complete-OWL-DL-consistency-checker">complete OWL DL consistency
checker</a> should not return <code>Unknown</code> on the relevant
<a href="#testConsistency">consistency</a>  
or <a href="#testInconsistency">inconsistency</a> tests.
</p>
<%--
<p>Many of the <a href="#testNonEntailment">
non-entailment tests</a> can be converted into 
<a href="#testConsistency">consistency tests</a> by negating the conclusions. 
Many of the 
<a href="#testEntailment">
entailment tests</a> can be converted into <a href="#testInconsistency">inconsistency tests</a> 
by negating the conclusions.
When this process is applied to an OWL Lite entailment or non-entailment,
the resulting consistent or inconsistent document is usually an 
<a href="#dfn-OWL-DL-document">OWL DL 
document</a>.
</p>
--%>
<%=head(1,"manifest","Manifest Files (Informative)")%>
<p>
The <code>Manifest</code> file follows the RDF schema developed
for the RDF Test Cases <%=ref("RDF Test Cases")%>.
</p>
<p>
This is augmented by a few new properties and types
which are declared in the OWL Test Ontology, found
at <a href="http://www.w3.org/2002/03owlt/testOntology">http://www.w3.org/2002/03owlt/testOntology</a>.
</p>
<p>
Specifically each test has its own <code>Manifest</code> file, and is identified from
the URI reference formed from the <code>Manifest</code> file's URL with a fragment <code>test</code>.
</p>
<p>
The test has one <code>rdf:type</code> explicit, and this is one of:
</p>
<dl>
<dt><code>otest:NotOwlFeatureTest</code></dt>
<dd><a href="#testBadOwl">A test for the incorrect use of the OWL namespace name.</a></dd>

<dt><code>otest:PositiveEntailmentTest</code></dt>
<dd><a href="#testEntailment">An entailment test.</a></dd>


<dt><code>otest:NegativeEntailmentTest</code></dt>
<dd><a href="#testNonEntailment">A non-entailment test.</a></dd>


<dt><code>otest:TrueTest</code></dt>
<dd><a href="#testTrue">A true test.</a></dd>


<dt><code>otest:OWLforOWLTest</code></dt>
<dd><a href="#testOWLforOWL">An OWL for OWL test.</a></dd>

<dt><code>otest:ConsistencyTest</code></dt>
<dd><a href="#testConsistency">A consistency test.</a></dd>

<dt><code>otest:InconsistencyTest</code></dt>
<dd><a href="#testInconsistency">An inconsistency test.</a></dd>
<dt><code>otest:ImportEntailmentTest</code></dt>
<dd><a href="#testImportEntailment">An import entailment test.</a></dd>
<dt><code>otest:ImportLevelTest</code></dt>
<dd><a href="#testImportLevel">An import level test.</a></dd>

</dl>
<p>Where <code>otest</code> is bound to 
<code>http://www.w3.org/2002/03owlt/testOntology#</code>
and <code>rtest</code> is bound to
<code>http://www.w3.org/2000/10/rdf-tests/rdfcore/testSchema#</code>.
</p>
<p>
The name of the original author of the test is shown using a
<code>dc:creator</code> property, see <%=ref("Dublin Core")%>.
</p>
<p>
A description of the test is given (using XHTML markup <%=ref("XHTML")%>)
as the value of the <code>rtest:description</code> property.
</p>
<p>
An issue, if any, from the OWL Issues list <%=ref("OWL Issues")%>, is
the value of a <code>rtest:issue</code> property.
</p>
<p>
An appropriate language feature, from the OWL namespace, if any, is
the value of the <code>otest:feature</code> property.
</p>
<p>
The input documents with the test data are found as the value of 
the <code>rtest:inputDocument</code> property or
as the value of both the
<code>rtest:premiseDocument</code> and
the 
<code>rtest:conclusionDocument</code>.
The support files for import entailment tests and import level tests are found
as the values of <code>otest:importedPremiseDocument</code>.
</p>
<p>
The conformance level associated with both files and tests
are given with the <code>otest:level</code> property.
The value for each document and test is one of
<code>otest:Full</code>, <code>otest:DL</code>,
<code>otest:Lite</code> or <code>otest:Other</code> (documents only).
</p>

<%@ include file="tests0.jsp" %>



<%=head(1,"editorialIssues","Editorial Issues (Informative)")%>
<p>
Last Call comments regarding any of these issues are welcome.
In particular, the existence of an issue in this section does
not constitute a promise that this will be addressed before
the Proposed Recommendation.</p>
<%=head(2,"editorialIssuesMachine","Arising from Automated Checking")%>
<p>
Most of these issues will be ignored.
</p>
<p>
<%=approved.getErrors()%>
<%=Manifest.loader.getErrors()%>
</p>
<%=head(2,"editorialStructure","Document Structure")%>
<p>
This document is too long.
The intend is to restructure it before Proposed Recommendation.
</p>
<p>
It is desirable to provide a PDF version that prints with 
correct page boundaries.
</p>
<%=head(2,"editorialTestRelated","Test Specific Issues")%>
<p>Many tests need to be changed or to have their syntactic level
changed in order to reflect recent decisions concerning
OWL DL syntax.</p>
<p >
Exhaustive coverage of OWL features is desired.
</p>
<p>
Tests reflecting more issue resolutions are desired.
</p>
<p>
<a href="http://lists.w3.org/Archives/Public/www-webont-wg/2003Mar/0140.html">
Defects</a> in the tests reported by Sean Bechhofer should be fixed.
</p>
<p>The TrueTests category needs to be used - there are a few tests that should
be changed. The OWL For OWL test category should be used, new tests need
to be created.</p>
<p>Many of the description logic tests should be changed into
consistency or inconsitency tests, to maximize usefulness for testing
consistency checkers.</p>
<p>Lite versions of many of the DL tests can be added, using the
techniques of the tests in C.2.2.</p>
<p>There are a number of outstanding ACTIONs in WebOnt telecon minutes,
or suggestions in the WebOnt e-mail for creation of tests to be added
to the test suite. Specifically concerning:
<a href="http://lists.w3.org/Archives/Public/www-webont-wg/2003Mar/0132.html">
datatypes</a>,
<a href="http://lists.w3.org/Archives/Public/www-webont-wg/2003Mar/0098.html">
owl:imports and its domain and range</a>,
<a href="http://lists.w3.org/Archives/Public/www-webont-wg/2003Feb/0193.html">
XMLLiteral</a>,
<a href="http://www.w3.org/2001/sw/WebOnt/ftf5.html">
language compliance levels</a>,
<a href="http://www.w3.org/2003/01/09-webont-irc#T09-43-01">
language compliance levels</a>,
<a href="http://www.w3.org/2003/01/09-webont-irc#T09-58-38">
owl:sameInstanceAs, owl:sameClassAs</a>,
and
<a href="http://lists.w3.org/Archives/Public/www-webont-wg/2003Mar/0044.html">
datatype properties in OWL Full</a>.
</p>
<p>Additional arithmetic tests can be added for OWL DL.</p>
<p>An infinite model test can be added for OWL Lite.</p>
<p>
In the extra credit section, the tests concerning prime factorization need fixing.
Other possible
tests may concern dominos and transitiveproperties with cardinality constraints
or may concern peculiar extensions of the OWL vocabulary.
</p>
<p>Other additional tests will be added as submitted by WG members and the community
during Last Call, and beyond.</p>
<%=head(2,"editorialIssuesHuman","Other")%>
<p>
The datatypes required by a test in the datatype theory
of a reasoner should be made explicit, and documentation
of the manifest file representation of this should be provided.</p>
<p>The
<a href="http://www.w3.org/2002/03owlt/testOntology"> 
test ontology</a> needs to be updated to reflect recent changes.
There should be corresponding additional
changes to the documentation of the manifest files.</p>
<p>
Can the descriptions of approved tests be quotes from other (normative) documents?
</p>
<p>
Check descriptions for appropriate links to other OWL specs.
</p>
<p>
Add automatic checking of filenames.
</p>


<hr />

<%=head(0,"acknowledgments","Acknowledgments (Informative)")%>
<p>
Jeremy Carroll thanks <a href="mailto:oreste@w3.org">Oreste Signore</a>, 
his host at
the <a href="http://www.w3c.it/">W3C Office in Italy</a> and 
<a href="http://www.isti.cnr.it">Istituto di Scienza e Tecnologie dell'Informazione 
"Alessandro Faedo"</a>, part of the
<a href="http://www.cnr.it">Consiglio Nazionale delle Ricerche</a>, where Jeremy is a 
visiting researcher.
</p>
<p>
The following people have contributed tests to this document: 
<% { String a[] = Manifest.authors();
    for (int i=0;i<a.length;i++) {
     if (a[i].endsWith("Carroll")) continue;
     if (a[i].endsWith("Roo")) continue;
%>
<%=a[i]%>,
<% }
} %>
and the editors.
</p>
<p>Ian Horrocks contributed to <a href="#conformance">the conformance</a> section of this document.
</p>
<p>
This document is the result of extensive discussions within the
<a href="http://www.w3.org/2001/sw/WebOnt/">Web Ontology Working Group 
</a>as a whole.  The members of this group working group included: 
Yasser al Safadi, Jean-Franois Baget,  James Barnette, Sean 
Bechhofer, Jonathan Borden,  Frederik Brysse, Stephen Buswell, Peter 
Crowther, Jos De Roo, David De Roure,  Mike Dean, Larry Eshelman, 
Jrme Euzenat, Dieter Fensel, Tim Finin, Nicholas Gibbins, Pat 
Hayes,  Jeff Heflin, Ziv Hellman, James Hendler,  Bernard Horan, 
Masahiro Hori, Ian Horrocks, Francesco Iannuzzelli, Mario Jeckle, 
Ruediger Klein,  Ora Lassila, Alexander Maedche, Massimo Marchiori, 
Deborah McGuinness, Libby Miller, Enrico Motta,  Leo Obrst, Laurent 
Olivry , Peter Patel-Schneider, Martin Pike, Marwan Sabbouh, Guus 
Schreiber, Shimizu Noboru,  Michael Sintek,  Michael Smith, Ned 
Smith, John Stanton,  Lynn Andrea Stein, Herman ter Horst,  Lynne R. 
Thompson,  David Trastour, Frank van Harmelen,  Raphael Volz, Evan 
Wallace, Christopher Welty, and John Yanosy.
</p>
<% if (editorsDraft) { %>
<%=head(1,"changes","Change Log")%>
<p class="todo">This section is only for WG review, 
and gives the changes since the 
<a href="http://www.w3.org/TR/2003/WD-owl-test-20030331/">
31st March version</a>.
</p>
<p>Added 
<a href=
"http://cvs.sourceforge.net/cgi-bin/viewcvs.cgi/jena/jena2/src/com/hp/hpl/jena/ontology/tidy/">
Jena2 OWL Syntax Checker 
</a>
into document creation tool.
Syntax error messages included in document.</p>
<p>Changed format of literals in document body in N3 to use
triple quote convention.</p>
<p>Added 
<a href="Manifest.rdf">Manifest</a> for all the tests as requested by Sean.</p>
<% } %>
<%=head(0,"references","References")%>




